/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ogmrip-audio-options.h"
#include "ogmrip-preferences.h"
#include "ogmrip-helper.h"
#include "ogmrip-plugin.h"
#include "ogmrip-gconf.h"

#include <glib/gi18n.h>
#include <glade/glade.h>

#define OGMRIP_GLADE_FILE "ogmrip/ogmrip-profile-editor.glade"
#define OGMRIP_GLADE_ROOT "audio-page"

#define OGMRIP_AUDIO_OPTIONS_DIALOG_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_AUDIO_OPTIONS_DIALOG, OGMRipAudioOptionsDialogPriv))

struct _OGMRipAudioOptionsDialogPriv
{
  GType container;
  GtkWidget *root;
  GtkWidget *codec_combo;
  GtkWidget *default_button;
  GtkWidget *quality_spin;
  GtkWidget *srate_combo;
  GtkWidget *channels_combo;
  GtkWidget *normalize_check;
  GtkListStore *codec_store;
};

static void
ogmrip_audio_options_dialog_reset (OGMRipAudioOptionsDialog *dialog)
{
  gint active;

  ogmrip_audio_options_dialog_set_codec (dialog, ogmrip_gconf_get_audio_codec_type (NULL));

  active = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_QUALITY, OGMRIP_DEFAULT_AUDIO_QUALITY);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->priv->quality_spin), active);

  active = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_SRATE, OGMRIP_DEFAULT_AUDIO_SRATE);
  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->srate_combo), active);

  active = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_CHANNELS, OGMRIP_DEFAULT_AUDIO_CHANNELS);
  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->channels_combo), active);

  active = ogmrip_preferences_get_bool (OGMRIP_GCONF_AUDIO_NORMALIZE, OGMRIP_DEFAULT_AUDIO_NORMALIZE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->normalize_check), active);
}

static void
ogmrip_audio_options_dialog_default_toggled (OGMRipAudioOptionsDialog *dialog)
{
  gboolean active;

  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button));
  gtk_widget_set_sensitive (dialog->priv->root, !active);
}

static void
ogmrip_audio_options_dialog_append_codec (GType type, const gchar *name, const gchar *description, OGMRipAudioOptionsDialog *dialog)
{
  GtkTreeIter iter;

  if (ogmrip_plugin_can_contain_audio (dialog->priv->container, type))
  {
    gtk_list_store_append (dialog->priv->codec_store, &iter);
    gtk_list_store_set (dialog->priv->codec_store, &iter, 0, description, 1, name, -1);
  }
}

static void
ogmrip_audio_options_dialog_populate_codec_combo (OGMRipAudioOptionsDialog *dialog)
{
  dialog->priv->container = ogmrip_gconf_get_container_type (NULL);
  ogmrip_plugin_foreach_audio_codec ((OGMRipPluginFunc) ogmrip_audio_options_dialog_append_codec, dialog);
}

static void
ogmrip_audio_options_dialog_construct_codec_combo (OGMRipAudioOptionsDialog *dialog)
{
  GtkCellRenderer *cell;
  gint n_codecs;

  dialog->priv->codec_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
  gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->priv->codec_combo), GTK_TREE_MODEL (dialog->priv->codec_store));
  g_object_unref (dialog->priv->codec_store);

  cell = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (dialog->priv->codec_combo), cell, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (dialog->priv->codec_combo), cell, "text", 0, NULL);

  n_codecs = ogmrip_plugin_get_n_audio_codecs ();
  gtk_widget_set_sensitive (dialog->priv->codec_combo, n_codecs > 0);
}

G_DEFINE_TYPE (OGMRipAudioOptionsDialog, ogmrip_audio_options_dialog, GTK_TYPE_DIALOG);

static void
ogmrip_audio_options_dialog_class_init (OGMRipAudioOptionsDialogClass *klass)
{
  g_type_class_add_private (klass, sizeof (OGMRipAudioOptionsDialogPriv));
}

static void
ogmrip_audio_options_dialog_init (OGMRipAudioOptionsDialog *dialog)
{
  GtkWidget *alignment, *reset;
  GladeXML *xml;

  dialog->priv = OGMRIP_AUDIO_OPTIONS_DIALOG_GET_PRIVATE (dialog);

  xml = glade_xml_new (OGMRIP_DATA_DIR "/" OGMRIP_GLADE_FILE, OGMRIP_GLADE_ROOT, NULL);
  if (!xml)
  {
    g_warning ("Could not find " OGMRIP_GLADE_FILE);
    return;
  }

  reset = gtk_button_new_with_label (_("Reset"));
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), reset);
  gtk_widget_show (reset);

  g_signal_connect_swapped (reset, "clicked", G_CALLBACK (ogmrip_audio_options_dialog_reset), dialog);

  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);

  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_window_set_title (GTK_WINDOW (dialog), _("Audio Track Options"));
  gtk_window_set_icon_from_stock (GTK_WINDOW (dialog), GTK_STOCK_PROPERTIES);

  alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 6, 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), alignment);
  gtk_widget_show (alignment);

  dialog->priv->default_button = gtk_check_button_new_with_label (_("Use default settings"));
  gtk_container_add (GTK_CONTAINER (alignment), dialog->priv->default_button);
  gtk_widget_show (dialog->priv->default_button);

  dialog->priv->root = glade_xml_get_widget (xml, OGMRIP_GLADE_ROOT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), dialog->priv->root, TRUE, TRUE, 0);
  gtk_widget_show (dialog->priv->root);

  g_signal_connect_swapped (dialog->priv->default_button, "toggled",
      G_CALLBACK (ogmrip_audio_options_dialog_default_toggled), dialog);

  dialog->priv->codec_combo = glade_xml_get_widget (xml, "audio-codec-combo");
  ogmrip_audio_options_dialog_construct_codec_combo (dialog);
  ogmrip_audio_options_dialog_populate_codec_combo (dialog);

  dialog->priv->quality_spin = glade_xml_get_widget (xml, "quality-spin");
  dialog->priv->srate_combo = glade_xml_get_widget (xml, "srate-combo");
  dialog->priv->channels_combo = glade_xml_get_widget (xml, "channels-combo");
  dialog->priv->normalize_check = glade_xml_get_widget (xml, "normalize-check");

  g_object_unref (xml);
}

GtkWidget *
ogmrip_audio_options_dialog_new (void)
{
  return g_object_new (OGMRIP_TYPE_AUDIO_OPTIONS_DIALOG, NULL);
}

void
ogmrip_audio_options_dialog_set_use_defaults (OGMRipAudioOptionsDialog *dialog, gboolean use_defaults)
{
  g_return_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button), use_defaults);
}

gboolean
ogmrip_audio_options_dialog_get_use_defaults (OGMRipAudioOptionsDialog *dialog)
{
  g_return_val_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog), FALSE);

  return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button));
}

void
ogmrip_audio_options_dialog_set_codec (OGMRipAudioOptionsDialog *dialog, GType codec)
{
  GtkTreeModel *model;
  GtkTreeIter iter;

  model = GTK_TREE_MODEL (dialog->priv->codec_store);
  if (gtk_tree_model_iter_children (model, &iter, NULL))
  {
    const gchar *name;
    gchar *str;

    name = ogmrip_plugin_get_audio_codec_name (codec);
    do
    {
      gtk_tree_model_get (model, &iter, 1, &str, -1);
      if (g_str_equal (str, name))
      {
        gtk_combo_box_set_active_iter (GTK_COMBO_BOX (dialog->priv->codec_combo), &iter);
        g_free (str);
        break;
      }
      g_free (str);
    }
    while (gtk_tree_model_iter_next (model, &iter));

    if (gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->priv->codec_combo)) < 0)
      gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->codec_combo), 0);
  }
}

GType
ogmrip_audio_options_dialog_get_codec (OGMRipAudioOptionsDialog *dialog)
{
  GType codec;
  GtkTreeIter iter;
  gchar *name;

  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->priv->codec_combo), &iter))
    return G_TYPE_NONE;

  gtk_tree_model_get (GTK_TREE_MODEL (dialog->priv->codec_store), &iter, 1, &name, -1);
  codec =  ogmrip_gconf_get_audio_codec_type (name);
  g_free (name);

  return codec;
}

void
ogmrip_audio_options_dialog_set_quality (OGMRipAudioOptionsDialog *dialog, guint quality)
{
  g_return_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog));

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->priv->quality_spin), quality);
}

gint
ogmrip_audio_options_dialog_get_quality (OGMRipAudioOptionsDialog *dialog)
{
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog), -1);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button)))
    quality = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_QUALITY, OGMRIP_DEFAULT_AUDIO_QUALITY);
  else
    quality = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dialog->priv->quality_spin));

  return quality;
}

void
ogmrip_audio_options_dialog_set_srate (OGMRipAudioOptionsDialog *dialog, guint srate)
{
  g_return_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog));

  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->srate_combo), srate);
}

gint
ogmrip_audio_options_dialog_get_srate (OGMRipAudioOptionsDialog *dialog)
{
  gint srate;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog), -1);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button)))
    srate = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_SRATE, OGMRIP_DEFAULT_AUDIO_SRATE);
  else
    srate = gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->priv->srate_combo));

  return srate;
}

void
ogmrip_audio_options_dialog_set_channels (OGMRipAudioOptionsDialog *dialog, guint channels)
{
  g_return_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog));

  gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->channels_combo), channels);
}

gint
ogmrip_audio_options_dialog_get_channels (OGMRipAudioOptionsDialog *dialog)
{
  gint channels;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog), -1);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button)))
    channels = ogmrip_preferences_get_int (OGMRIP_GCONF_AUDIO_CHANNELS, OGMRIP_DEFAULT_AUDIO_CHANNELS);
  else
    channels = gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->priv->channels_combo));

  return channels;
}

void
ogmrip_audio_options_dialog_set_normalize (OGMRipAudioOptionsDialog *dialog, gboolean normalize)
{
  g_return_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog));

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->normalize_check), normalize);
}

gboolean
ogmrip_audio_options_dialog_get_normalize (OGMRipAudioOptionsDialog *dialog)
{
  gboolean normalize;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_OPTIONS_DIALOG (dialog), FALSE);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->default_button)))
    normalize = ogmrip_preferences_get_bool (OGMRIP_GCONF_AUDIO_NORMALIZE, OGMRIP_DEFAULT_AUDIO_NORMALIZE);
  else
    normalize = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->normalize_check));

  return normalize;
}

